@@ -33,7 +33,7 @@ pub fn to_name_type(x: &str) -> anyhow::Result<InnerNameType> {
     if bytes.len() > DEVICE_NAME_LEN {
         anyhow::bail!("Maximum name length ({} chars) exceeded.", DEVICE_NAME_LEN);
     }
-    (&mut name[..bytes.len()]).copy_from_slice(bytes);
+    name[..bytes.len()].copy_from_slice(bytes);
     Ok(name)
 }
 
@@ -65,12 +65,11 @@ pub struct TriggerClockInfoRow {
 }
 
 struct SerialHandler {
-    device: String,
     icr1_and_prescaler: Option<TopAndPrescaler>,
     version_check_done: bool,
     qi: u8,
     queries: BTreeMap<u8, chrono::DateTime<chrono::Utc>>,
-    ser: Option<tokio_serial::SerialStream>,
+    ser: tokio_serial::SerialStream,
     outq: Receiver<Cmd>,
     vquery_time: chrono::DateTime<chrono::Utc>,
     last_time: chrono::DateTime<chrono::Utc>,
@@ -92,22 +91,54 @@ pub enum Cmd {
 }
 
 impl SerialHandler {
-    fn new(
-        device: String,
+    async fn new(
+        device_path: String,
         outq: Receiver<Cmd>,
         on_new_model_cb: ClockModelCallback,
         triggerbox_data_tx: Option<Sender<TriggerClockInfoRow>>,
         max_acceptable_measurement_error: Duration,
+        assert_device_name: NameType,
+        baud_rate: u32,
     ) -> Result<Self> {
         let now = chrono::Utc::now();
         let vquery_time = now + Duration::seconds(1);
+
+        debug!("Opening device at path {}", device_path);
+
+        let (ser, name) = match tokio::time::timeout(
+            std::time::Duration::from_millis(5000),
+            serial_handshake(&device_path, baud_rate),
+        )
+        .await
+        {
+            Ok(r) => r,
+            Err(elapsed) => Err(elapsed).map_err(anyhow::Error::from),
+        }
+        .with_context(|| format!("opening device {}", device_path))?;
+
+        if let Some(name) = &name {
+            let name_str = String::from_utf8_lossy(name);
+            debug!("Connected to device named \"{}\".", name_str);
+        } else {
+            debug!("Connected to unnamed device.");
+        }
+
+        if assert_device_name.is_some() && name != assert_device_name {
+            anyhow::bail!(
+                "Found name {}, but expected {}. ({:?} vs {:?}.)",
+                name_display(&name),
+                name_display(&assert_device_name),
+                name,
+                assert_device_name,
+            );
+        }
+
         Ok(Self {
-            device,
             icr1_and_prescaler: None,
             version_check_done: false,
             qi: 0,
             queries: BTreeMap::new(),
-            ser: None,
+            ser,
             outq,
             vquery_time, // wait 1 second before first version query
             last_time: vquery_time + Duration::seconds(1), // and 1 second after version query
@@ -120,48 +151,18 @@ impl SerialHandler {
     }
 
     async fn write(&mut self, buf: &[u8]) -> tokio::io::Result<()> {
-        if let Some(ref mut ser) = self.ser {
-            trace!("sending: \"{}\"", String::from_utf8_lossy(buf));
-            for byte in buf.iter() {
-                trace!("sending byte: {}", byte);
-            }
-            AsyncWriteExt::write_all(ser, buf).await?;
-        } else {
-            panic!("serial device null")
+        trace!("sending: \"{}\"", String::from_utf8_lossy(buf));
+        for byte in buf.iter() {
+            trace!("sending byte: {}", byte);
         }
+        AsyncWriteExt::write_all(&mut self.ser, buf).await?;
         Ok(())
     }
 
-    async fn run(
-        mut self: SerialHandler,
-        query_dt: std::time::Duration,
-        assert_device_name: NameType,
-        baud_rate: u32,
-    ) -> Result<()> {
+    /// Run forever, handling interaction with the triggerbox hardware device.
+    async fn run_forever(mut self: SerialHandler, query_dt: std::time::Duration) -> Result<()> {
         let query_dt = Duration::from_std(query_dt)?;
 
-        let (ser, name) = serial_handshake(&self.device, baud_rate)
-            .await
-            .map_err(anyhow::Error::from)
-            .with_context(|| format!("opening device {}", self.device))?;
-
-        if assert_device_name.is_some() && name != assert_device_name {
-            anyhow::bail!(
-                "Found name {}, but expected {}. ({:?} vs {:?}.)",
-                name_display(&name),
-                name_display(&assert_device_name),
-                name,
-                assert_device_name,
-            );
-        }
-
-        // ser.set_timeout(std::time::Duration::from_millis(10))?;
-
-        self.ser = Some(ser);
-        if let Some(name) = &name {
-            let name_str = String::from_utf8_lossy(name);
-            debug!("connected to device named \"{}\"", name_str);
-        }
         let mut now = chrono::Utc::now();
 
         let connect_time = now;
@@ -173,97 +174,82 @@ impl SerialHandler {
         loop {
             // handle new commands
             if self.version_check_done {
-                loop {
-                    match tokio::time::timeout(
-                        std::time::Duration::from_millis(0),
-                        self.outq.recv(),
-                    )
-                    .await
-                    {
-                        Ok(opt_cmd_tup) => {
-                            if let Some(cmd_tup) = opt_cmd_tup {
-                                debug!("got command {:?}", cmd_tup);
-                                match cmd_tup {
-                                    Cmd::TopAndPrescaler(new_value) => {
-                                        self._set_top_and_prescaler(new_value).await?;
-                                    }
-                                    Cmd::StopPulsesAndReset => {
-                                        debug!(
-                                        "will reset counters. dropping outstanding info requests."
-                                    );
-                                        self.allow_requesting_clock_sync = false;
-                                        self.queries.clear();
-                                        self.past_data.clear();
-                                        (self.on_new_model_cb)(None);
-                                        self.write(b"S0").await?;
-                                    }
-                                    Cmd::StartPulses => {
-                                        self.allow_requesting_clock_sync = true;
-                                        self.write(b"S1").await?;
-                                    }
-                                    Cmd::SetDeviceName(name) => {
-                                        let computed_crc = format!(
-                                            "{:X}",
-                                            arduino_udev::CRC_MAXIM.checksum(&name)
-                                        );
-                                        trace!("computed CRC: {:?}", computed_crc);
+                while let Ok(opt_cmd_tup) =
+                    tokio::time::timeout(std::time::Duration::from_millis(0), self.outq.recv())
+                        .await
+                {
+                    if let Some(cmd_tup) = opt_cmd_tup {
+                        debug!("got command {:?}", cmd_tup);
+                        match cmd_tup {
+                            Cmd::TopAndPrescaler(new_value) => {
+                                self._set_top_and_prescaler(new_value).await?;
+                            }
+                            Cmd::StopPulsesAndReset => {
+                                debug!("will reset counters. dropping outstanding info requests.");
+                                self.allow_requesting_clock_sync = false;
+                                self.queries.clear();
+                                self.past_data.clear();
+                                (self.on_new_model_cb)(None);
+                                self.write(b"S0").await?;
+                            }
+                            Cmd::StartPulses => {
+                                self.allow_requesting_clock_sync = true;
+                                self.write(b"S1").await?;
+                            }
+                            Cmd::SetDeviceName(name) => {
+                                let computed_crc =
+                                    format!("{:X}", arduino_udev::CRC_MAXIM.checksum(&name));
+                                trace!("computed CRC: {:?}", computed_crc);
 
-                                        self.write(b"N=").await?;
-                                        self.write(&name).await?;
-                                        self.write(computed_crc.as_bytes()).await?;
-                                    }
-                                    Cmd::SetAOut((volts1, volts2)) => {
-                                        fn volts_to_dac(volts: f64) -> u16 {
-                                            // Convert voltage to fraction and clamp.
-                                            let frac = (volts / 4.096).clamp(0.0, 1.0);
-                                            // Compute integer DAC value.
-                                            let val: u16 = (frac * 4095.0).round() as u16;
-                                            val
-                                        }
-                                        let val1 = volts_to_dac(volts1);
-                                        let val2 = volts_to_dac(volts2);
-
-                                        self.write(b"O=").await?;
-                                        self.write(&val1.to_le_bytes()).await?;
-                                        self.write(&val2.to_le_bytes()).await?;
-                                        self.write(b"x").await?;
-
-                                        // Now wait for return value.
-                                        tokio::time::sleep(std::time::Duration::from_millis(50))
-                                            .await;
-
-                                        let mut buf = vec![0; 100];
-                                        let len = self.ser.as_mut().unwrap().read(&mut buf).await?;
-                                        let buf = &buf[..len];
-                                        debug!("AOUT ignoring values: {:?}", buf);
-                                    }
-                                    Cmd::SetLedPulse(val) => {
-                                        let buf = val.encode();
-                                        self.write(&buf[..]).await?;
-                                    }
+                                self.write(b"N=").await?;
+                                self.write(&name).await?;
+                                self.write(computed_crc.as_bytes()).await?;
+                            }
+                            Cmd::SetAOut((volts1, volts2)) => {
+                                fn volts_to_dac(volts: f64) -> u16 {
+                                    // Convert voltage to fraction and clamp.
+                                    let frac = (volts / 4.096).clamp(0.0, 1.0);
+                                    // Compute integer DAC value.
+                                    let val: u16 = (frac * 4095.0).round() as u16;
+                                    val
                                 }
-                            } else {
-                                // no more commands, sender hung up
-                                info!("exiting run loop");
-                                return Ok(());
+                                let val1 = volts_to_dac(volts1);
+                                let val2 = volts_to_dac(volts2);
+
+                                self.write(b"O=").await?;
+                                self.write(&val1.to_le_bytes()).await?;
+                                self.write(&val2.to_le_bytes()).await?;
+                                self.write(b"x").await?;
+
+                                // Now wait for return value.
+                                tokio::time::sleep(std::time::Duration::from_millis(50)).await;
+
+                                let mut buf = vec![0; 100];
+                                let len = self.ser.read(&mut buf).await?;
+                                let buf = &buf[..len];
+                                debug!("AOUT ignoring values: {:?}", buf);
+                            }
+                            Cmd::SetLedPulse(val) => {
+                                let buf = val.encode();
+                                self.write(&buf[..]).await?;
                             }
                         }
-                        Err(_elapsed) => {
-                            // timeout
-                            break;
-                        }
+                    } else {
+                        // no more commands, sender hung up
+                        info!("exiting run loop");
+                        return Ok(());
                     }
                 }
             }
 
             // get all pending data
-            if let Some(ref mut ser) = self.ser {
+            {
                 // TODO: this could be made (much) more efficient. Right
                 // now, we wake up every timeout duration and run the whole
                 // cycle when no byte arrives.
                 match tokio::time::timeout(
                     std::time::Duration::from_millis(10),
-                    ser.read(&mut read_buf),
+                    self.ser.read(&mut read_buf),
                 )
                 .await
                 {
@@ -281,8 +267,6 @@ impl SerialHandler {
                     }
                     Err(_elapsed) => {}
                 }
-            } else {
-                unreachable!();
             }
             // handle pending data
             buf = self._h(buf).await?;
@@ -531,7 +515,7 @@ fn test_fit_time_model() {
 pub async fn run_triggerbox(
     on_new_model_cb: ClockModelCallback,
     device: String,
-    cmd: Receiver<Cmd>,
+    outq: Receiver<Cmd>,
     triggerbox_data_tx: Option<Sender<TriggerClockInfoRow>>,
     query_dt: std::time::Duration,
     assert_device_name: NameType,
@@ -540,15 +524,15 @@ pub async fn run_triggerbox(
     let baud_rate = 115_200;
     let triggerbox = SerialHandler::new(
         device,
-        /*raw_tx,*/ cmd,
+        outq,
         on_new_model_cb,
         triggerbox_data_tx,
         Duration::from_std(max_acceptable_measurement_error).unwrap(),
+        assert_device_name,
+        baud_rate,
     )
-    .unwrap();
-    triggerbox
-        .run(query_dt, assert_device_name, baud_rate)
-        .await
+    .await?;
+    triggerbox.run_forever(query_dt).await
 }
 
 fn get_rate(rate_ideal: f64, prescaler: Prescaler) -> (u16, f64) {
